//
//  FileSystemWatcher.m
//  Pulse
//
//  Created by Matt Attaway on 1/9/14.
//  Copyright (c) 2014 Zen of the Monkey. All rights reserved.
//

#import "PLSFileEvent.h"
#import "FileSystemWatcher.h"


void fsevents_callback(ConstFSEventStreamRef streamRef,
                       void * userData,
                       size_t numEvents,
                       void * eventPaths,
                       const FSEventStreamEventFlags eventFlags[],
                       const FSEventStreamEventId eventIds[]);

@implementation FileSystemWatcher

-(void)dealloc
{
    [self stopWatching];
}

- (void)watchPath:(NSString*)path target:(NSObject< FileSystemWatcherDelegate > *)target
{
    listener = target;
    NSArray* pathsToWatch = [NSArray arrayWithObjects:path, nil];
    void* appPointer = (__bridge void *)self;
    FSEventStreamContext context = {0, appPointer, NULL, NULL, NULL};
    NSTimeInterval latency = 1.0;

    stream = FSEventStreamCreate( NULL,
                                  &fsevents_callback,
                                  &context,
                                  (__bridge CFArrayRef) pathsToWatch,
                                  kFSEventStreamEventIdSinceNow,
                                  (CFAbsoluteTime) latency,
                                  kFSEventStreamCreateFlagFileEvents | kFSEventStreamCreateFlagNoDefer
                                  );

    FSEventStreamScheduleWithRunLoop(stream, CFRunLoopGetCurrent(), kCFRunLoopDefaultMode);
}

-(void)fileEvent:(PLSFileEvent*)event
{
    [listener fileEvent:event];
}

- (void)startWatching
{
    if(stream)
    {
        FSEventStreamStart(stream);
    }
}

- (void)pauseWatching
{
    if(stream)
    {
        FSEventStreamStop(stream);
    }
}

- (void)stopWatching
{
	FSEventStreamStop(stream);
	FSEventStreamUnscheduleFromRunLoop(stream,
									   CFRunLoopGetCurrent(),
									   kCFRunLoopDefaultMode);
	FSEventStreamInvalidate(stream);
	FSEventStreamRelease(stream);
}

@end

void fsevents_callback(ConstFSEventStreamRef streamRef,
                       void * userData,
                       size_t numEvents,
                       void * eventPaths,
                       const FSEventStreamEventFlags eventFlags[],
                       const FSEventStreamEventId eventIds[])
{
    uint32 modifiedFiles = kFSEventStreamEventFlagItemCreated | kFSEventStreamEventFlagItemRemoved | kFSEventStreamEventFlagItemRenamed |kFSEventStreamEventFlagItemModified;
    
    FileSystemWatcher * watcher = (__bridge FileSystemWatcher *)userData;
    NSLog(@"File Changed! %zu", numEvents);
    char **paths = eventPaths;
    NSMutableArray* fileActions = [[NSMutableArray alloc] init];
    NSMutableArray* files = [[NSMutableArray alloc] init];
    
    for( int i = 0; i < numEvents; i++ )
    {
        if(eventFlags[i] & modifiedFiles)
        {
            [files addObject:[NSString stringWithUTF8String:paths[i]]];
            [fileActions addObject:[[NSNumber alloc] initWithUnsignedInt:(unsigned int)eventFlags[i]]];
            NSLog(@"Change %llu in %s, flags %x\n", eventIds[i], paths[i], (unsigned int)eventFlags[i]);
        }
        else
        {
            NSLog(@"Ignoring %llu in %s, flags %x\n", eventIds[i], paths[i], (unsigned int)eventFlags[i]);
        }
    }

    if([files count] > 0)
    {
        PLSFileEvent* event = [[PLSFileEvent alloc] init];
        event.fileActions = fileActions;
        event.files = files;
        [watcher fileEvent:event];
    }
}