//
//  PLSConnection2Controller.m
//  Pulse
//
//  Created by Matt Attaway on 3/3/14.
//  Copyright (c) 2014 Zen of the Monkey. All rights reserved.
//

#import "PLSConnectionController.h"
#import "PLSOverseer.h"

@interface PLSConnectionController ()

@end

@implementation PLSConnectionController


- (id)initWithWindow:(NSWindow *)window
{
    self = [super initWithWindow:window];
    if (self) {
        // Initialization code here.
    }
    return self;
}


- (void)windowDidLoad
{
    [super windowDidLoad];
}


- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView
{
    return [self.connections count];
}


- (NSView*)tableView:(NSTableView *)tableView viewForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
    if(row == -1)
    {
        return nil;
    }
    
    PLSOverseer *os = [self.connections objectAtIndex:row];
    
    NSTableCellView* cell = [tableView makeViewWithIdentifier:@"connectionCell" owner:self];
    [cell.textField setStringValue:os.port];
    [cell.imageView setImage:[self getStatusImage:os]];

    return cell;
}


- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    [self updateFields];
}


- (void)updateFields
{
    NSInteger row = [self.tableView selectedRow];
    
    PLSOverseer* os = self.connections[row];
    [self.portField   setStringValue:os.port];
    [self.userField   setStringValue:os.user];
    [self.pathField   setStringValue:os.path];
    [self.clientField setStringValue:os.client];
    
    self.enableAutoSubmitBox.state = os.enableAutoSubmit;
    
    [self.statusIcon setImage:[self getStatusImage:os]];
    if(os.disabled)
    {
        [self.statusField setStringValue:@"Updates paused"];
        [self.statusButton setTitle:@"Resume Updates"];
    }
    else if(os.connectionDown)
    {
        [self.statusField setStringValue:@"Server unavailable"];
        [self.statusButton setTitle:@"Pause Updates"];
    }
    else if (os.needsLogin)
    {
        [self.statusField setStringValue:@"Login to resume updates"];
        [self.statusButton setTitle:@"Pause Updates"];
    }
    else
    {
        [self.statusField setStringValue:@"Connected"];
        [self.statusButton setTitle:@"Pause Updates"];
    }
}


- (NSImage*)getStatusImage:(PLSOverseer*)os
{
    if(os.disabled || os.connectionDown)
    {
        return [NSImage imageNamed:@"NSStatusUnavailable"];
    }
    else if (os.needsLogin)
    {
        return [NSImage imageNamed:@"NSStatusPartiallyAvailable"];
    }
    else
    {
        return [NSImage imageNamed:@"NSStatusAvailable"];
    }
}


- (IBAction)updateSelectedConnection:(id)sender
{
    NSString *identifier = [sender identifier];
    PLSOverseer* os = self.connections[[self.tableView selectedRow]];
    
    if([sender isKindOfClass:[NSTextField class]])
    {
        [os setValue:[(NSTextField*)sender stringValue] forKey:identifier];
    }
    else if([sender isKindOfClass:[NSButton class]])
    {
        if([identifier isEqual:@"changeUpdateState"])
        {
            os.disabled = !os.disabled;
        }
        else if([identifier isEqual:@"enableAutoSubmit"])
        {
            os.enableAutoSubmit = [(NSButton*)sender state];
        }
    }
    
    // make sure the fields are all up-to-date
    [self updateFields];
    
    // update the table as well
    NSIndexSet* rowSet = [NSIndexSet indexSetWithIndex:[self.tableView selectedRow]];
    NSIndexSet* colSet = [NSIndexSet indexSetWithIndex:0];
    [self.tableView reloadDataForRowIndexes:rowSet columnIndexes:colSet];
}


- (IBAction)add:(id)sender
{
    PLSOverseer* os = [[PLSOverseer alloc] init];
    [self.connections addObject:os];
    [self.tableView reloadData];

    NSIndexSet* indexSet = [NSIndexSet indexSetWithIndex:[self.connections count]-1];
    [self.tableView selectRowIndexes:indexSet byExtendingSelection:false];
}


- (IBAction)remove:(id)sender
{
    NSInteger row = [self.tableView selectedRow];
    [self.tableView abortEditing];
    
    if(row != -1)
    {
        [self.connections removeObjectAtIndex:row];
        [self.tableView reloadData];
    }
    
    NSIndexSet* indexSet = [NSIndexSet indexSetWithIndex:[self.connections count]-1];
    [self.tableView selectRowIndexes:indexSet byExtendingSelection:false];
}


-(void)showWindow:(id)sender
{
    [super showWindow:sender];
    [self.tableView reloadData];
    
    for( PLSOverseer* os in self.connections )
    {
        [os pauseTracking];
    }
    
    NSIndexSet* indexSet = [NSIndexSet indexSetWithIndex:0];
    [self.tableView selectRowIndexes:indexSet byExtendingSelection:false];
}


- (void)windowWillClose:(NSNotification *)notification
{
    // on the off chance we didn't set them while editing, update the selected object on the way out
    PLSOverseer* sos = [self.connections objectAtIndex:[self.tableView selectedRow]];
    sos.port   = [self.portField   stringValue];
    sos.user   = [self.userField   stringValue];
    sos.path   = [self.pathField   stringValue];
    sos.client = [self.clientField stringValue];
    
    // write out all the connection to the preferences
    NSMutableArray* osSettings = [[NSMutableArray alloc] init];
    for(PLSOverseer* os in self.connections)
    {
        [osSettings addObject:[os getSettings]];
    }
    
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:osSettings forKey:@"connections"];
    [defaults synchronize];
}


- (IBAction)launchSelectFile:(id)sender {

    NSOpenPanel* panel = [NSOpenPanel openPanel];
    
    [panel setCanChooseFiles:NO];
    [panel setAllowsMultipleSelection:NO];
    [panel setCanChooseDirectories:YES];
    [panel setCanCreateDirectories:YES];
    [panel setPrompt:@"Select"];
    [panel setDirectoryURL:[NSURL URLWithString:[self.pathField stringValue]]];
    
    if ( [panel runModal] == NSOKButton )
    {
        NSArray *files = [panel URLs];
        [self.pathField   setStringValue:[files[0] path]];
        [self updateSelectedConnection:self.pathField];
    }
}

@end

