//
//  PLSOverseer.h
//  Pulse
//
//  Created by Matt Attaway on 1/20/14.
//  Copyright (c) 2014 Zen of the Monkey. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "FileSystemWatcher.h"

@class P4Message;
@class PLSOverseer;
@class PLSFileEvent;

@protocol PLSOverseerDelegate <NSObject>

@required
-(void)needPassword:(PLSOverseer*)os;
-(void)needPasswordNow:(PLSOverseer*)os;
-(void)connectionDown:(PLSOverseer*)os;
-(void)connectionUp:(PLSOverseer*)os;
-(void)loggedIn;
@end

@interface PLSOverseer : NSObject <FileSystemWatcherDelegate, NSUserNotificationCenterDelegate>

// user set properties
@property NSString* port;
@property NSString* user;
@property NSString* client;
@property NSString* path;
@property BOOL      enableAutoSubmit;
@property BOOL      disabled;

// automatically configured properties
@property NSString* charset;
@property NSInteger highChange;

// overseer state properties
@property BOOL needsLogin;
@property BOOL connectionDown;

// overseer thread state properties
@property BOOL syncing;
@property BOOL reconciling;
@property BOOL lookingForConnection;
@property BOOL lookingForTicket;

@property NSObject<PLSOverseerDelegate >* delegate;

-(id)initWithSettings:(NSString*)port user:(NSString*)user client:(NSString*)client path:(NSString*)path highChange:(NSInteger)highChange charset:(NSString*)charset enableAutoSubmit:(BOOL)enableAutoSubmit disabled:(BOOL)disabled;
-(id)initWithDictionary:(NSDictionary*)dict;


// for NSUserNotificationCenterDelegate
-(BOOL)userNotificationCenter:(NSUserNotificationCenter*)center shouldPresentNotification:(NSUserNotification*)notification;

-(NSDictionary*)getSettings;
-(void)startTracking;
-(void)pauseTracking;
-(P4Message*)login:(NSString*)password;

@end
